/*
* Программа кодирования букв Эсперанто с диакритикой.
* Разработано при поддержке проекта Язык Эсперанто - плановый язык, доступный каждому <языкэсперанто.рф>.
* Разработано при поддержке проекта Техническая библиотека "Основы техники" <основытехники.рф>.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

enum deystviye
{
	DEKODIROVAT,
	KODIROVAT
};

int main(int argc, char * argv[])
{
	enum deystviye rezhim_raboty = DEKODIROVAT;
	
	char* istochnik = NULL;
	char* naznacheniye = NULL;
		
	if ((argc < 2) || (argc >= 2 && (strcmp(argv[1], "-s") == 0 || strcmp(argv[1], "-h") == 0 || strcmp(argv[1], "-?") == 0 || strcmp(argv[1], "--spravka") == 0 || strcmp(argv[1], "--helpo") == 0)))
	{
		//Вывод справочной информации об использовании программы
		printf("Использование: espkodi [ПАРАМЕТР] ИСТОЧНИК [НАЗНАЧЕНИЕ]\n");
		printf("Преобразует псевдокоды в файле ИСТОЧНИК в буквы Эсперанто с диакритикой\n");
		printf("и записывает результат в файл НАЗНАЧЕНИЕ.\n");
		printf("Кодировка символов входного и выходного файлов - юникод УТФ-8 (UTF-8).\n");
		printf("Если файл НАЗНАЧЕНИЕ не указан - выводит результат на экран.\n\n");
		printf("Возможные значения параметров:\n");
		printf("-?, -s, --spravka, -h, --helpo - вывод данной справочной информации\n");
		printf("-v, --versio, -r, --redakcio  - вывод информации о версии программы\n");
		printf("-d, -m, --malkodi - преобразовать псевдокоды в буквы с диакритикой\n");
		printf("-k, --kodi - преобразовать буквы с диакритикой в псевдокоды\n\n");
		printf("По умолчанию используется режим \"декодировать\" - преобразование\n");
		printf("псевдокодов в буквы Эсперанто с диакритикой.\n");		
		return 0;
	}
	else if ((argc == 2) && (strcmp(argv[1], "-v") == 0 || strcmp(argv[1], "-r") == 0 || strcmp(argv[1], "--versio") == 0 || strcmp(argv[1], "--redakcio") == 0))
	{
		printf("Кодирование букв Эсперанто с диакритикой.\n");
		printf("Сборка 2024.04.30.\n");
		printf("Разработано при поддержке проекта\n");
		printf("  Язык Эсперанто - плановый язык, доступный каждому <языкэсперанто.рф>\n");
		printf("Разработано при поддержке проекта\n");
		printf("  Техническая библиотека \"Основы техники\" <основытехники.рф>\n");
		
		return 0;		
	}
	else if (argc >= 2)
	{
		if (argv[1][0] == '-')
		{
			//Задано значение параметра
			if (strcmp(argv[1], "-d") == 0 || strcmp(argv[1], "-m") == 0 || strcmp(argv[1], "--malkodi") == 0)
			{
				rezhim_raboty = DEKODIROVAT;			
			}
			else if (strcmp(argv[1], "-k") == 0 || strcmp(argv[1], "--kodi") == 0)
			{
				rezhim_raboty = KODIROVAT;
			}
			else
			{
				printf("Неверный формат вызова: указан неподдерживаемый параметр.\n");
				printf("Для получения справки введите espkodi -?.\n");
				return 1;
			}
			
			if (argc < 3)
			{
				printf("Неверный формат вызова: не указано имя файла для обработки.\n");
				printf("Для получения справки введите espkodi -?\n");
				return 2;
			}
			
			istochnik = argv[2];
			
			if (argc == 4) 
			{
				naznacheniye = argv[3];
			}
			
			if (argc > 4)
			{
				printf("Неверный формат вызова: слишком много параметров.\n");
				printf("Для получения справки введите espkodi -?\n");
				return 3;
			}
		}
		else
		{
			//Параметры не заданы. Используется режим работы по умолчанию			
			istochnik = argv[1];
			
			if (argc == 3) 
			{
				naznacheniye = argv[2];
			}
			
			if (argc > 3)
			{
				printf("Неверный формат вызова: слишком много параметров.\n");
				printf("Для получения справки введите espkodi -?\n");
				return 3;
			}
		}	
	}
	
	if (rezhim_raboty == DEKODIROVAT)
	{
		printf("Декодирование псевдокодов в буквы Эсперанто с диакритикой...\n");
	}
	else if (rezhim_raboty == KODIROVAT)
	{
		printf("Кодирование букв Эсперанто с диакритикой в псевдокоды...\n");
	}
	
	//Открываем файл для чтения
	FILE *file_istochnik = fopen(istochnik, "r");
	
	if (file_istochnik == NULL)
	{
		printf("Не удалось открыть файл для чтения.\n");
		return 4;
	}
	
	fseek(file_istochnik, 0, SEEK_END);
	
	size_t razmer_dannih = ftell(file_istochnik);
	
	if (razmer_dannih == 0) {
		printf("Исходный файл пуст.\n");
		return 5;		
	}
	
	//Выделяем память для хранения данных + 1 байт для нулевого символа
	size_t razmer_bufera = razmer_dannih + 1;
	
	char* bufer = malloc(razmer_bufera);	
	if (bufer == NULL)
	{	
		fclose(file_istochnik);	
		printf("Не удалось выделить память для чтения файла.\n");
		return 6;
	}
	
	fseek(file_istochnik, 0, SEEK_SET);
	
	//Читаем данные из файла-источника
	fread(bufer, razmer_dannih, 1, file_istochnik);
	
	//Закрываем файл-источник
	fclose(file_istochnik);
	
	//Устанавливаем нулевой символ в конце буфера чтения
	bufer[razmer_dannih] = '\0';
	
	//Кодирование/декодирование букв Эсперанто с диакритикой
	//Если размер исходного файла 1 байт - декодировать нечего
	if (razmer_dannih >= 2)
	{
		for (size_t i = 0; i < (razmer_dannih - 1); i++){
			char tekSimv = bufer[i];
			char sledSimv = bufer[i + 1];
			
			//Ищем пары кодов символов и, при необходимости, заменяем
			if (rezhim_raboty == DEKODIROVAT)
			{
				if (bufer[i] == (char)0x43 && bufer[i+1] == (char)0x78)
				{
					//Cx
					bufer[i] = (char)0xc4;
					bufer[i+1] = (char)0x88;
					i++;
					continue;
				}
				if (bufer[i] == (char)0x43 && bufer[i+1] == (char)0x58)
				{
					//CX
					bufer[i] = (char)0xc4;
					bufer[i+1] = (char)0x88;
					i++;
					continue;
				}
				if (bufer[i] == (char)0x63 && bufer[i+1] == (char)0x78)
				{
					//cx
					bufer[i] = (char)0xc4;
					bufer[i+1] = (char)0x89;
					i++;
					continue;
				}
				if (bufer[i] == (char)0x47 && bufer[i+1] == (char)0x78)
				{
					//Gx
					bufer[i] = (char)0xc4;
					bufer[i+1] = (char)0x9c;
					i++;
					continue;
				}
				if (bufer[i] == (char)0x47 && bufer[i+1] == (char)0x58)
				{
					//GX
					bufer[i] = (char)0xc4;
					bufer[i+1] = (char)0x9c;
					i++;
					continue;
				}
				if (bufer[i] == (char)0x67 && bufer[i+1] == (char)0x78)
				{
					//gx
					bufer[i] = (char)0xc4;
					bufer[i+1] = (char)0x9d;
					i++;
					continue;
				}
				if (bufer[i] == (char)0x48 && bufer[i+1] == (char)0x78)
				{
					//Hx
					bufer[i] = (char)0xc4;
					bufer[i+1] = (char)0xa4;
					i++;
					continue;
				}
				if (bufer[i] == (char)0x48 && bufer[i+1] == (char)0x58)
				{
					//HX
					bufer[i] = (char)0xc4;
					bufer[i+1] = (char)0xa4;
					i++;
					continue;
				}
				if (bufer[i] == (char)0x68 && bufer[i+1] == (char)0x78)
				{
					//hx
					bufer[i] = (char)0xc4;
					bufer[i+1] = (char)0xa5;
					i++;
					continue;
				}
				if (bufer[i] == (char)0x4a && bufer[i+1] == (char)0x78)
				{
					//Jx
					bufer[i] = (char)0xc4;
					bufer[i+1] = (char)0xb4;
					i++;
					continue;
				}
				if (bufer[i] == (char)0x4a && bufer[i+1] == (char)0x58)
				{
					//JX
					bufer[i] = (char)0xc4;
					bufer[i+1] = (char)0xb4;
					i++;
					continue;
				}
				if (bufer[i] == (char)0x6a && bufer[i+1] == (char)0x78)
				{
					//jx
					bufer[i] = (char)0xc4;
					bufer[i+1] = (char)0xb5;
					i++;
					continue;
				}
				if (bufer[i] == (char)0x53 && bufer[i+1] == (char)0x78)
				{
					//Sx
					bufer[i] = (char)0xc5;
					bufer[i+1] = (char)0x9c;
					i++;
					continue;
				}
				if (bufer[i] == (char)0x53 && bufer[i+1] == (char)0x58)
				{
					//SX
					bufer[i] = (char)0xc5;
					bufer[i+1] = (char)0x9c;
					i++;
					continue;
				}
				if (bufer[i] == (char)0x73 && bufer[i+1] == (char)0x78)
				{
					//sx
					bufer[i] = (char)0xc5;
					bufer[i+1] = (char)0x9d;
					i++;
					continue;
				}
				if (bufer[i] == (char)0x55 && bufer[i+1] == (char)0x78)
				{
					//Ux
					bufer[i] = (char)0xc5;
					bufer[i+1] = (char)0xac;
					i++;
					continue;
				}
				if (bufer[i] == (char)0x55 && bufer[i+1] == (char)0x58)
				{
					//UX
					bufer[i] = (char)0xc5;
					bufer[i+1] = (char)0xac;
					i++;
					continue;
				}
				if (bufer[i] == (char)0x75 && bufer[i+1] == (char)0x78)
				{
					//ux
					bufer[i] = (char)0xc5;
					bufer[i+1] = (char)0xad;
					i++;
					continue;
				}
			}
			else if (rezhim_raboty == KODIROVAT)
			{
				if (bufer[i] == (char) 0xc4 && bufer[i+1] == (char)0x88)
				{
					//Cx
					bufer[i] = (char)0x43;
					bufer[i+1] = (char)0x78;
					i++;
					continue;
				}
				if (bufer[i] == (char)0xc4 && bufer[i+1] == (char)0x89)
				{
					//cx
					bufer[i] = (char)0x63;
					bufer[i+1] = (char)0x78;
					i++;
					continue;
				}
				if (bufer[i] == (char)0xc4 && bufer[i+1] == (char)0x9c)
				{
					//Gx
					bufer[i] = (char)0x47;
					bufer[i+1] = (char)0x78;
					i++;
					continue;
				}
				if (bufer[i] == (char)0xc4 && bufer[i+1] == (char)0x9d)
				{
					//gx
					bufer[i] = (char)0x67;
					bufer[i+1] = (char)0x78;
					i++;
					continue;
				}
				if (bufer[i] == (char)0xc4 && bufer[i+1] == (char)0xa4)
				{
					//Hx
					bufer[i] = (char)0x48;
					bufer[i+1] = (char)0x78;
					i++;
					continue;
				}
				if (bufer[i] == (char)0xc4 && bufer[i+1] == (char)0xa5)
				{
					//hx
					bufer[i] = (char)0x68;
					bufer[i+1] = (char)0x78;
					i++;
					continue;
				}
				if (bufer[i] == (char)0xc4 && bufer[i+1] == (char)0xb4)
				{
					//Jx
					bufer[i] = (char)0x4a;
					bufer[i+1] = (char)0x78;
					i++;
					continue;
				}
				if (bufer[i] == (char)0xc4 && bufer[i+1] == (char)0xb5)
				{
					//jx
					bufer[i] = (char)0x6a;
					bufer[i+1] = (char)0x78;
					i++;
					continue;
				}
				if (bufer[i] == (char)0xc5 && bufer[i+1] == (char)0x9c)
				{
					//Sx
					bufer[i] = (char)0x53;
					bufer[i+1] = (char)0x78;
					i++;
					continue;
				}
				if (bufer[i] == (char)0xc5 && bufer[i+1] == (char)0x9d)
				{
					//sx
					bufer[i] = (char)0x73;
					bufer[i+1] = (char)0x78;
					i++;
					continue;
				}
				if (bufer[i] == (char)0xc5 && bufer[i+1] == (char)0xac)
				{
					//Ux
					bufer[i] = (char)0x55;
					bufer[i+1] = (char)0x78;
					i++;
					continue;
				}
				if (bufer[i] == (char)0xc5 && bufer[i+1] == (char)0xad)
				{
					//ux
					bufer[i] = (char)0x75;
					bufer[i+1] = (char)0x78;
					i++;
					continue;
				}
			}
		}
	}
	
	if (naznacheniye != NULL)
	{
		//Записываем результат работы в выходной файл
		//Открываем файл для записи
		FILE *file_naznacheniye = fopen(naznacheniye, "w");
		
		if (file_naznacheniye == NULL)
		{
			printf("Не удалось открыть файл для записи.\n");
			return 7;
		}
		
		//Записываем данные в файл
		fwrite(bufer, razmer_dannih, 1, file_naznacheniye);
		
		//Закрываем файл
		fclose(file_naznacheniye);
		
		printf("Успешно.\n");
	}
	else
	{
		//Если не указан файл назначения - выводим результат на экран
		printf("%s\n", bufer);
	}
	
	//Освобождаем память
	free(bufer);
	bufer = NULL;
	
	return 0;
}
